
# line 38 "parse.y"

/*
**      Include files.
**      --------------
*/

#include "way.h"
#include <stdio.h>
#include <ctype.h>
#include <strings.h>
#include "parse.h"

/*
**      Forward references.
**      -------------------
*/

static int yylex();
static void yyerror();

/*
**	Flag to handle context-dependency, info fields are parsed differently.
**	----------------------------------------------------------------------
*/

static int parsing_info = 0;

/*
**      Variables to control parser input and actions.
**      ----------------------------------------------
*/

#ifdef __STDC__
static int  (*parser_input)(void);
static void (*parser_action)(NS_DECLARATION_PTR ns_decl);
#else
static int  (*parser_input)();
static void (*parser_action)();
#endif

/*
**      External functions.
**      -------------------
*/

#ifdef __STDC__
extern char *xmalloc(int size);
extern char *xrealloc(char *ptr, int size);
extern int keyword_type(const char *keyword);
#else
extern char *xmalloc();
extern char *xrealloc();
extern int keyword_type();
#endif

/*
**      External functions that don't like too much checking.
**      -----------------------------------------------------
*/

extern void fatal(/* function, format, arg1, arg2, ... */);
extern void error(/* function, format, arg1, arg2, ... */);
extern void xfree(/* any-pointer */);


# line 112 "parse.y"
typedef union  {
    char *           name_type;
    ADDRESS_PTR      address_type;
    SERVICE_LIST_PTR service_type;
    INFO_LIST_PTR    info_type;
} YYSTYPE;
# define PROVIDES 257
# define NAME 258
# define FLAG 259
# define AMBIGUOUS 260
# define UNKNOWN 261
# define NUMERIC 262
# define ALPHA 263
# define NUMBER 264
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern short yyerrflag;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
YYSTYPE yylval, yyval;
# define YYERRCODE 256

# line 338 "parse.y"



/*
**	Macros and variables for yylex lookahead mechanism.
**	---------------------------------------------------
*/

#define GETC      (cbufp == cbuf ? (*parser_input)() : *--cbufp)
#define UNGETC(c) (*cbufp++ = (c))

static int *cbufp;
static int cbuf[16];

/*
**	Just to provide a slightly better diagnostics.
**	----------------------------------------------
*/

static int line_number;


/*
**	The parser initialization function.
**	-----------------------------------
*/

#ifdef __STDC__
void init_parser(int (*input)(void), void (*action)(NS_DECLARATION_PTR decl))
#else
void init_parser(input, action)
int (*input)();
void (*action)();
#endif
{
    /* Initialize parser input and action function variables. */

    parser_input = input;
    parser_action = action;

    /* Initialize the line number counter. */

    line_number = 1;

    /* Initialize the yylex lookahead variables. */

    cbufp = cbuf;
}

/*
**	The parser error reporting function.
**	------------------------------------
*/

static void yyerror(string)
char *string;
{
    fatal("yyparse", "%s in line %d", string, line_number);
}


/*
**	The parser lexical analyzer.
**	----------------------------
*/

static char *name_terminators = "(),; \t\f\r\b\n/-=";

#define IS_TERMINATOR(c) (!(strchr(name_terminators, c) == (char *) 0))

static int yylex()
{
    int name_buf_len;
    char *name_buf;
    register int c, c1;
    register int i;
    int inside_comment;
    int is_quoted      = 0;
    int is_a_number    = 0;
    int is_hexadecimal = 0;

    c = GETC;
    while (1) {
	switch (c) {
	    case ' ' :			/* Skip all the white stuff. */
	    case '\t':
	    case '\f':
	    case '\r':
	    case '\b':
		c = GETC;
		break;

	    case '\n':			/* Increment the line number. */
		line_number++;
		c = GETC;
		break;

	    case '"':
		is_quoted = 1;
					/* INTENTIONALLY FALL THROUGH */
	    case '\\':
		c = GETC;
		goto get_name;
		break;

	    case '/':			/* Maybe a start of comment. */
		c1 = GETC;
		if (c1 != '*') {
		    UNGETC(c1);
		    goto get_name;	/* Not a comment, get name. */
		}

		c = GETC;
		inside_comment = 1;	/* We are inside a comment now. */
		while (inside_comment) {/* Skip to its end. */
		    if (c == '*') {
			while (c == '*')
			    c = GETC;

			if (c == '/') {	/* We are no longer inside it. */
			    inside_comment = 0;
			    c = GETC;
			}
		    } else if (c == '\n') {
			line_number++;	/* Continue taking care of line no. */
			c = GETC;
		    } else if (c == EOF) {
			error("yylex",
				"unexpected EOF inside comment at line %d",
				line_number);
			break;
		    } else
			c = GETC;	/* Continue skipping. */
		}
		break;

	    case '-':			/* Maybe the PROVIDES token. */
		c1 = GETC;
		if (c1 != '>') {
		    UNGETC(c1);
		    goto get_name;	/* No, it isn't, get name. */
		}

		return PROVIDES;	/* Yes, it is. */

	    case '(':			/* These are token themselves. */
	    case ')':
	    case ',':
	    case ';':
		return c;

	    case '=':			/* If we are parsing info */
		if (parsing_info)	/*  this is a token.      */
		    return c;
		else
		    goto get_name;
		break;

	    case EOF:			/* End of file. */
		return 0;

get_name:
	    default:

		/* Set the initial buffer length to 16. */

		name_buf_len = 16;
		name_buf = xmalloc(name_buf_len + 1);

		/* If we are parsing info we must check if we get a number. */

		if (parsing_info) {
		    is_a_number = 1;
		    is_hexadecimal = 0;
		}

		i = 0;
carry_on:
		do {

		    /* If buffer is full, make it bigger. */

		    if (i == name_buf_len) {
			name_buf_len *= 2;
			name_buf = xrealloc(name_buf, name_buf_len + 1);
		    }

		    /* Check if it is good for a number. */

		    if (parsing_info && is_a_number && !isdigit(c))
			switch (i) {
			    case 0:
				if (c != '-')
				    is_a_number = 0;
				break;
			    case 1:
				if ((c == 'x' || c == 'X') &&
							    name_buf[0] == '0')
				    is_hexadecimal = 1;
				else
				    is_a_number = 0;
				break;
			    default:
				if (is_hexadecimal &&
					strchr("abcdefABCDEF", c) == (char *) 0)
				    is_a_number = 0;
			}    /* switch */

		    /* Add current character to buffer. */

		    name_buf[i++] = c;

		    /* Get next character, if it is a back-slash
		       or we are inside a quoted string the normal
		       termination conditions don't apply.
		    */

		    c = GETC;
		    if (c == '\\') {
			c = GETC;
			continue;
		    }
		    if (is_quoted)
			if (c == EOF)
			    error("yylex",
			       "unexpected EOF inside quoted string at line %d",
				line_number);
			else if (c != '"')
			    continue;
			else
			    break;
		} while (c != EOF && !IS_TERMINATOR(c));

		/* Make sure it was a real terminator, that is one of
		   normal terminators or a comment delimiter or a
		   PROVIDES token or, if we are parsing the additiona
		   informations, an equal sign.  If not a real terminator
		   then carry on reading the name.
		*/

		if (c == '/' || c == '-') {
		    c1 = GETC;
		    UNGETC(c1);
		    if (c == '/' && c1 != '*' || c == '-' && c1 != '>')
			goto carry_on;
		}
		else if (c == '=' && !parsing_info)
		    goto carry_on;

		/* If it is not a quoted string put back the terminator. */

		if (!is_quoted)
		    UNGETC(c);

		/* Null-terminate the name buffer and return it
		   trying to guess the appropriate token.
		*/

		name_buf[i] = '\0';
		yylval.name_type = name_buf;

		if (parsing_info) {
		    if (is_a_number)
			return NUMBER;
		    if (!is_quoted)
			return keyword_type(name_buf);
		}
		return NAME;
	    }	/* switch */
    }		/* while */
}
short yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
	};
# define YYNPROD 26
# define YYLAST 49
short yyact[]={

  31,  32,  33,  34,  35,  36,  37,  18,  29,  11,
  19,  20,   5,   6,  26,  25,  14,  38,  23,  15,
  24,  24,   7,  17,  16,   9,  22,  12,   2,  30,
   8,  13,  10,   3,   4,   1,   0,   0,   0,   0,
  21,   0,   0,   0,   0,   0,   0,  27,  28 };
short yypact[]={

-1000,-246,-1000,-244, -18,-1000,-249,-1000, -28,-1000,
 -21,-1000,-252,-1000,-249,-1000, -23,-1000,-1000, -46,
 -47,-1000,-252,-1000,-252,-256,-258, -24,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000 };
short yypgo[]={

   0,  35,  34,  33,  32,  25,  30,  23,  24,  29,
  28,  27,  26 };
short yyr1[]={

   0,   1,   1,  10,   3,  11,   3,   6,   6,   5,
  12,   5,   8,   8,   2,   4,   7,   7,   7,   9,
   9,   9,   9,   9,   9,   9 };
short yyr2[]={

   0,   0,   2,   4,   1,   0,   5,   1,   3,   1,
   0,   5,   1,   3,   1,   1,   1,   3,   3,   1,
   1,   1,   1,   1,   1,   1 };
short yychk[]={

-1000,  -1, -10,  -3,  -2, 258, 257,  40,  -6,  -5,
  -4, 258, -11,  59,  44,  40,  -8,  -7, 259, 262,
 263,  -5, -12,  41,  44,  61,  61,  -8,  -7, 264,
  -9, 258, 259, 260, 261, 262, 263, 264,  41 };
short yydef[]={

   1,  -2,   2,   0,   4,  14,   0,   5,   0,   7,
   9,  15,   0,   3,   0,  10,   0,  12,  16,   0,
   0,   8,   0,   6,   0,   0,   0,   0,  13,  17,
  18,  19,  20,  21,  22,  23,  24,  25,  11 };
#ifndef lint
static char yaccpar_sccsid[] = "@(#)yaccpar	4.1	(Berkeley)	2/11/83";
#endif not lint

#
# define YYFLAG -1000
# define YYERROR goto yyerrlab
# define YYACCEPT return(0)
# define YYABORT return(1)

/*	parser for yacc output	*/

#ifdef YYDEBUG
int yydebug = 0; /* 1 for debugging */
#endif
YYSTYPE yyv[YYMAXDEPTH]; /* where the values are stored */
int yychar = -1; /* current input token number */
int yynerrs = 0;  /* number of errors */
short yyerrflag = 0;  /* error recovery flag */

yyparse() {

	short yys[YYMAXDEPTH];
	short yyj, yym;
	register YYSTYPE *yypvt;
	register short yystate, *yyps, yyn;
	register YYSTYPE *yypv;
	register short *yyxi;

	yystate = 0;
	yychar = -1;
	yynerrs = 0;
	yyerrflag = 0;
	yyps= &yys[-1];
	yypv= &yyv[-1];

 yystack:    /* put a state and value onto the stack */

#ifdef YYDEBUG
	if( yydebug  ) printf( "state %d, char 0%o\n", yystate, yychar );
#endif
		if( ++yyps> &yys[YYMAXDEPTH] ) { yyerror( "yacc stack overflow" ); return(1); }
		*yyps = yystate;
		++yypv;
		*yypv = yyval;

 yynewstate:

	yyn = yypact[yystate];

	if( yyn<= YYFLAG ) goto yydefault; /* simple state */

	if( yychar<0 ) if( (yychar=yylex())<0 ) yychar=0;
	if( (yyn += yychar)<0 || yyn >= YYLAST ) goto yydefault;

	if( yychk[ yyn=yyact[ yyn ] ] == yychar ){ /* valid shift */
		yychar = -1;
		yyval = yylval;
		yystate = yyn;
		if( yyerrflag > 0 ) --yyerrflag;
		goto yystack;
		}

 yydefault:
	/* default state action */

	if( (yyn=yydef[yystate]) == -2 ) {
		if( yychar<0 ) if( (yychar=yylex())<0 ) yychar = 0;
		/* look through exception table */

		for( yyxi=yyexca; (*yyxi!= (-1)) || (yyxi[1]!=yystate) ; yyxi += 2 ) ; /* VOID */

		while( *(yyxi+=2) >= 0 ){
			if( *yyxi == yychar ) break;
			}
		if( (yyn = yyxi[1]) < 0 ) return(0);   /* accept */
		}

	if( yyn == 0 ){ /* error */
		/* error ... attempt to resume parsing */

		switch( yyerrflag ){

		case 0:   /* brand new error */

			yyerror( "syntax error" );
		yyerrlab:
			++yynerrs;

		case 1:
		case 2: /* incompletely recovered error ... try again */

			yyerrflag = 3;

			/* find a state where "error" is a legal shift action */

			while ( yyps >= yys ) {
			   yyn = yypact[*yyps] + YYERRCODE;
			   if( yyn>= 0 && yyn < YYLAST && yychk[yyact[yyn]] == YYERRCODE ){
			      yystate = yyact[yyn];  /* simulate a shift of "error" */
			      goto yystack;
			      }
			   yyn = yypact[*yyps];

			   /* the current yyps has no shift onn "error", pop stack */

#ifdef YYDEBUG
			   if( yydebug ) printf( "error recovery pops state %d, uncovers %d\n", *yyps, yyps[-1] );
#endif
			   --yyps;
			   --yypv;
			   }

			/* there is no state on the stack with an error shift ... abort */

	yyabort:
			return(1);


		case 3:  /* no shift yet; clobber input char */

#ifdef YYDEBUG
			if( yydebug ) printf( "error recovery discards char %d\n", yychar );
#endif

			if( yychar == 0 ) goto yyabort; /* don't discard EOF, quit */
			yychar = -1;
			goto yynewstate;   /* try again in the same state */

			}

		}

	/* reduction by production yyn */

#ifdef YYDEBUG
		if( yydebug ) printf("reduce %d\n",yyn);
#endif
		yyps -= yyr2[yyn];
		yypvt = yypv;
		yypv -= yyr2[yyn];
		yyval = yypv[1];
		yym=yyn;
			/* consult goto table to find next state */
		yyn = yyr1[yyn];
		yyj = yypgo[yyn] + *yyps + 1;
		if( yyj>=YYLAST || yychk[ yystate = yyact[yyj] ] != -yyn ) yystate = yyact[yypgo[yyn]];
		switch(yym){
			
case 3:
# line 160 "parse.y"
{
		    NS_DECLARATION_PTR tmp;
		    register SERVICE_LIST_PTR sscan, stmp;
		    register INFO_LIST_PTR iscan, itmp;

		    /* Build the final structure. */

		    tmp = (NS_DECLARATION_PTR) xmalloc(sizeof(NS_DECLARATION));
		    tmp->address = yypvt[-3].address_type;
		    tmp->service_list = yypvt[-1].service_type;

		    /* Pass the whole structure to user function. */

		    (*parser_action)(tmp);

		    /* Release memory space: this looks quite complicated
		       and I wont even try to clarify it with comments.
		       Use of alloca() would simplify things a lot, but I
		       don't know how it works on all the funny machines
		       around here (PC, Mac, m68k systems for which only
		       brain damaged C libraries are provided etc.)
		    */

		    xfree(tmp->address->name);
		    iscan = tmp->address->info;
		    while (iscan) {
			xfree(iscan->name);
			itmp = iscan;
			iscan = iscan->next;
			xfree(itmp);
		    }
		    xfree(tmp->address);
		    sscan = tmp->service_list;
		    while (sscan) {
			xfree(sscan->name);
			iscan = sscan->info;
			while (iscan) {
			    xfree(iscan->name);
			    itmp = iscan;
			    iscan = iscan->next;
			    xfree(itmp);
			}
			stmp = sscan;
			sscan = sscan->next;
			xfree(stmp);
		    }
		    xfree(tmp);
		} break;
case 4:
# line 216 "parse.y"
{
		    yypvt[-0].address_type->info = NULL_INFO_LIST_PTR;
		    yyval.address_type = yypvt[-0].address_type;
		} break;
case 5:
# line 220 "parse.y"
{ parsing_info = 1; } break;
case 6:
# line 221 "parse.y"
{
		    parsing_info = 0;
		    yypvt[-4].address_type->info = yypvt[-1].info_type;
		    yyval.address_type = yypvt[-4].address_type;
		} break;
case 7:
# line 233 "parse.y"
{
		    yyval.service_type = yypvt[-0].service_type;
		} break;
case 8:
# line 237 "parse.y"
{
		    yypvt[-0].service_type->next = yypvt[-2].service_type;
		    yyval.service_type = yypvt[-0].service_type;
		} break;
case 9:
# line 249 "parse.y"
{
		    yypvt[-0].service_type->info = NULL_INFO_LIST_PTR;
		    yyval.service_type = yypvt[-0].service_type;
		} break;
case 10:
# line 253 "parse.y"
{ parsing_info = 1; } break;
case 11:
# line 254 "parse.y"
{
		    parsing_info = 0;
		    yypvt[-4].service_type->info = yypvt[-1].info_type;
		    yyval.service_type = yypvt[-4].service_type;
		} break;
case 12:
# line 267 "parse.y"
{
		    yyval.info_type = yypvt[-0].info_type;
		} break;
case 13:
# line 271 "parse.y"
{
		    yypvt[-0].info_type->next = yypvt[-2].info_type;
		    yyval.info_type = yypvt[-0].info_type;
		} break;
case 14:
# line 278 "parse.y"
{
		    ADDRESS_PTR tmp;

		    tmp = (ADDRESS_PTR) xmalloc(sizeof(ADDRESS_ITEM));
		    tmp->name = yypvt[-0].name_type;
		    yyval.address_type = tmp;
		} break;
case 15:
# line 288 "parse.y"
{
		    SERVICE_LIST_PTR tmp;
	
		    tmp = (SERVICE_LIST_PTR) xmalloc(sizeof(SERVICE_LIST_ITEM));
		    tmp->name = yypvt[-0].name_type;
		    tmp->next = NULL_SERVICE_LIST_PTR;
		    yyval.service_type = tmp;
		} break;
case 16:
# line 299 "parse.y"
{
		    INFO_LIST_PTR tmp;

		    tmp = (INFO_LIST_PTR) xmalloc(sizeof(INFO_LIST_ITEM));
		    tmp->name = yypvt[-0].name_type;
		    tmp->next = NULL_INFO_LIST_PTR;
		    yyval.info_type = tmp;
		} break;
case 17:
# line 308 "parse.y"
{
		    INFO_LIST_PTR tmp;

		    tmp = (INFO_LIST_PTR) xmalloc(sizeof(INFO_LIST_ITEM));
		    tmp->name = yypvt[-2].name_type;
		    tmp->value = yypvt[-0].name_type;
		    tmp->next = NULL_INFO_LIST_PTR;
		    yyval.info_type = tmp;
		} break;
case 18:
# line 318 "parse.y"
{
		    INFO_LIST_PTR tmp;

		    tmp = (INFO_LIST_PTR) xmalloc(sizeof(INFO_LIST_ITEM));
		    tmp->name = yypvt[-2].name_type;
		    tmp->value = yypvt[-0].name_type;
		    tmp->next = NULL_INFO_LIST_PTR;
		    yyval.info_type = tmp;
		} break; 
		}
		goto yystack;  /* stack new state and value */

	}
